# -*- coding: utf-8 -*-
"""
Payment History service, records modification done on payments
"""
import logging

from endi_payment.interfaces import IPaymentArchiveService
from endi_payment.models import EndiPaymentHistory


class HistoryLogService(object):
    """
    History service that's only logging the actions in a log file.

    To be used in dev mode
    """
    def __init__(self, context, request):
        self.request = request
        self.logger = logging.getLogger("endi_payment")
        self.archive_service = request.find_service(
            IPaymentArchiveService
        )

    def record(self, action, invoice, payment):
        self.logger.debug("Recording an action")
        self.logger.debug("Action : %s" % action)
        self.logger.debug("User : %s" % self.request.user)
        self.logger.debug("Invoice id : %s" % invoice.id)


class HistoryDBService(HistoryLogService):
    """
    Service logging datas in a database using a specific connexion
    """

    def get_previous_entry_hash(self, dbsession):
        """
        Produce a hash of the csv representation of the previous history entry

        :param obj dbsession: The current database session
        :rtype: str
        """
        previous_entry = dbsession.query(
            EndiPaymentHistory
        ).order_by(EndiPaymentHistory.id).first()

        if previous_entry:
            result = previous_entry.get_hash()
        else:
            result = ""
        return result

    def record(self, action, invoice, payment):
        self.logger.info("Recording an action")
        self.logger.info("Action : %s" % action)
        self.logger.info("User : %s" % self.request.user)
        self.logger.info("Invoice id : %s" % invoice.id)
        self.logger.info("It's production mode")

        from endi_payment.database import LocalSessionContext
        with LocalSessionContext() as dbsession:
            previous_entry_hash = self.get_previous_entry_hash(dbsession)
            record = EndiPaymentHistory(
                action_type=action,
                payment_id=payment.id,
                mode=payment.mode,
                amount=payment.amount,
                bank_remittance_id=payment.bank_remittance_id,
                date=payment.date,
                invoice_id=invoice.id,
                invoice_pdf_file_hash=invoice.pdf_file_hash,
                bank_cg=payment.bank.compte_cg,
                tva_value=payment.tva.value,
                user_login=self.request.user.login.login,
                previous_entry_hash=previous_entry_hash,
            )
            dbsession.add(record)
            dbsession.flush()
            archive_seal = self.archive_service.archive(record)
            if archive_seal is not None:
                dbsession.add(archive_seal)
            else:
                self.logger.debug(u"Not archive seal was produced")

        self.logger.info(u"Recorded")



# -*- coding: utf-8 -*-
"""
Endi storage services
"""
import datetime
import hashlib
import os
import logging

from endi_payment.models import EndiPaymentArchiveSeal


class DefaultArchiveService(object):
    def __init__(self, context, request):
        self.request = request
        self.logger = logging.getLogger("endi_payment")

    def archive(self, history):
        self.logger.debug(u"Archiving history item {}".format(history.id))
        id_key = u"No persistent archive system has been used"
        result = EndiPaymentArchiveSeal(
            remote_identification_key=id_key,
            endi_payment_history_id=history.id
        )
        return result

    def is_archived(self, history):
        return False


class FileArchiveService(object):
    """
    FileStorageService simply logs the history in a file
    """
    settings_key = "endi_payment_archive_storage_path"

    def __init__(self, context, request):
        self.request = request
        self.storage_path = self.request.registry.settings[self.settings_key]
        today = datetime.date.today()
        self.filename = "payment_storage_{}_{}.csv".format(
            today.year, today.month
        )
        self.filepath = os.path.join(self.storage_path, self.filename)

    def archive(self, history):
        """
        Archive the given history entry

        :returns: A Sha1 sum of the output file content
        :rtype: str
        """
        with open(self.filepath, 'a') as fbuf:
            fbuf.write(history.serialize())

        with open(self.filepath, 'rb') as fbuf:
            id_key = hashlib.sha1(fbuf.read()).hexdigest()

        result = EndiPaymentArchiveSeal(
            remote_identification_key=id_key,
            endi_payment_history_id=history.id
        )
        return result

    def is_archived(self, history):
        """
        Check if the payment history entry has been archived

        :param obj history: The EndiPaymentHistory instance
        :rtype: bool
        """
        from endi_payment.database import HistorySessionFactory
        session = HistorySessionFactory()
        query = session.query(EndiPaymentArchiveSeal.id)
        query = query.filter(
            EndiPaymentArchiveSeal.endi_payment_history_id == history.id
        )
        return query.count() > 0

    @classmethod
    def check_settings(cls, settings):
        """
        Check the settings contains the endi_payment_archive_storage_path if
        this given service is configured in the ini file

        :raises: KeyError if the key is missing
        :raises: Exception if the directory doesn't exist
        """
        if cls.settings_key not in settings:
            raise KeyError(
                u"You should configure {} in your .ini file".format(
                    cls.settings_key
                )
            )

        storage_path = settings[cls.settings_key]
        if not os.path.isdir(storage_path):
            raise Exception(
                u"Invalid storage path {}".format(storage_path)
            )



# -*- coding: utf-8 -*-
from zope.interface import Interface


class IPaymentRecordHistoryService(Interface):
    """
    History manipulation tool iinterface for Payment action log
    """

    def record_action(self, action, invoice, payment):
        """
        History manipulation tool for Payment registration
        """
        pass


class IPaymentArchiveService(Interface):
    """
    Archive service is used to archive payment datas and secure its content
    integrity with the help of third-party services
    """

    def archive(self, history):
        """
        Stores a single history item

        :param obj history: A EndiPaymentHistory instance
        :rtype: class:`endi_payment.models.EndiPaymentArchiveSeal
        """
        pass

    def is_archived(self, history):
        """
        Check if the given history item has been archived remotely

        :param obj history: A EndiPaymentHistory instance
        :rtype: bool
        """
        pass
